#include "parser.h"

Tree::Tree(const QVector<QVariant> &data, Tree *parent)
    : itemData(data), itemParent(parent) {}

Tree::~Tree() {
    for (auto node : itemChildren) {
        itemParent = nullptr;
        delete node;
    }
}

Tree* Tree::child(int row) {
    if (row < 0 || row >= itemChildren.size()) {
        return nullptr;
    }

    return itemChildren.at(row);
}

void Tree::appendChild(Tree *item) {
    itemChildren.append(item);
}

int Tree::rowCount() const {
    return itemChildren.size();
}

int Tree::columnCount() const {
    return itemData.size();
}

QVariant Tree::data(int column) const {
    if (column < 0 || column >= itemData.size())
            return QVariant();
        return itemData.at(column);
}

bool Tree::setData(int column, QVariant value) {
    if (column < 0 || column >= itemData.size()) {
        return false;
    }
    itemData[column] = value;
    return true;
}

Tree* Tree::parent() const {
    return itemParent;
}

int Tree::rowInParent() const {
    if (itemParent) {
        return itemParent->itemChildren.indexOf(const_cast<Tree*>(this));
    }
    return 0;
}

void Tree::removeChild() {
    itemChildren.pop_back();
}

void Tree::removeChild(int row) {
    delete itemChildren[row];
    itemChildren.remove(row);
}

bool Tree::removeChilds(int begin, int end) {
    for (int i = begin; i < end; ++i) {
        delete itemChildren[begin];
        itemChildren.remove(begin);
    }

    return true;
}


XmlParser::XmlParser(QObject* parent)
    : QAbstractItemModel(parent), rootItem(new Tree({tr("")})), xmlReader(new QXmlStreamReader()), focused(), focusedFont()
{}



QVariant XmlParser::data(const QModelIndex& index, int role) const {
    if (!index.isValid()) {
        return QVariant();
    }
    if (role == Qt::FontRole && index == focused) {
        return focusedFont;
    }
    if (role != Qt::DisplayRole) {
        return QVariant();
    }
    Tree *item = static_cast<Tree*>(index.internalPointer());
    return item->data(index.column());
}

bool XmlParser::setData(const QModelIndex &index, const QVariant &value, int role) {
    if (!index.isValid()) {
        return false;
    }
    Tree *item = static_cast<Tree*>(index.internalPointer());
    if (role == Qt::EditRole) {
        return item->setData(index.column(), value);
    }

    if (role == Qt::FontRole) {
        focused = index;
        focusedFont = value.value<QFont>();
        return true;
    }
    return false;
}

Qt::ItemFlags XmlParser::flags(const QModelIndex& index) const {
    if (!index.isValid()) {
        return Qt::NoItemFlags;
    }
    return QAbstractItemModel::flags(index);
}

QVariant XmlParser::headerData(int section, Qt::Orientation orientation, int role) const {
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
        return rootItem->data(section);
    }

    return QVariant();

}


QModelIndex XmlParser::index(int row, int column, const QModelIndex& parent) const {
    Tree* parentItem;
    if (!parent.isValid()) {
        parentItem = rootItem;
    } else {
        parentItem = static_cast<Tree*>(parent.internalPointer());
    }
    Tree *childItem = parentItem->child(row);
    if (childItem) {
        return createIndex(row, column, childItem);
    }
    return QModelIndex();
}

QModelIndex XmlParser::parent(const QModelIndex& child) const {
    if (!child.isValid()) {
        return QModelIndex{};
    }
    Tree *childItem = static_cast<Tree*>(child.internalPointer());
    Tree *parentItem = childItem->parent();
    if (parentItem == rootItem || childItem == rootItem || !parentItem) {
        return QModelIndex();
    }
    return createIndex(parentItem->rowInParent(), 0, parentItem);
}


int XmlParser::rowCount(const QModelIndex& parent) const {
    Tree *parentItem;
    if (parent.column() > 0) {
        return 0;
    }
    if (!parent.isValid()) {
        parentItem = rootItem;
    }else {
        parentItem = static_cast<Tree*>(parent.internalPointer());
    }
    return parentItem->rowCount();

}

int XmlParser::columnCount(const QModelIndex& parent) const {
    if (parent.isValid()) {
        return static_cast<Tree*>(parent.internalPointer())->columnCount();
    }
    return rootItem->columnCount();
}


void XmlParser::LoadFile(const QString& fileName) {
    QFile file{fileName};

    QStack<int> counterStack;
    QStack<int> layerStack;

    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "Fail to open";
    }
    xmlReader->setDevice(&file);

    auto fileNode = new Tree{{fileName}, rootItem};

    int currentLayer = 0;


    rootItem->appendChild(fileNode);
    Tree* currentNode = fileNode;
    while (!xmlReader->atEnd() && !xmlReader->hasError()) {
        QXmlStreamReader::TokenType token = xmlReader->readNext();

        if (token == QXmlStreamReader::StartElement) {
            ++currentLayer;

            auto text = xmlReader->name().toString();

                if (layerStack.isEmpty() || currentLayer != layerStack.top()) {
                    counterStack.append(0);
                    layerStack.append(currentLayer);
                }

            auto node = new Tree({text}, currentNode);
            currentNode->appendChild(node);
            currentNode = node;
        }

        if (token == QXmlStreamReader::Characters) {
            auto node = new Tree({xmlReader->text().toString()}, currentNode);
            currentNode->appendChild(node);
        }

        if (token == QXmlStreamReader::EndElement) {
            --currentLayer;
            currentNode = currentNode->parent();
            auto text = xmlReader->name().toString();
            if (text == "array" && currentLayer + 1 != layerStack.top()) {
                counterStack.pop();
                layerStack.pop();
            }
        }
    }

    if (xmlReader->hasError()) {
        rootItem->removeChild();
        delete fileNode;
        QMessageBox message(QMessageBox::Critical, tr("Parsing failed"),
                            tr("Can't add this file"),
                            QMessageBox::Ok);
        message.exec();
    }

    file.close();
}


void XmlParser::closeAll() {
    removeRows(0, rootItem->rowCount(), createIndex(-1, -1, nullptr));
}

void XmlParser::close() {
    auto fileTreeNode = static_cast<Tree*>(focused.internalPointer());
    if (fileTreeNode) {
        removeRow(fileTreeNode->rowInParent(), focused.parent());
        QFont font;
        font.setBold(true);
        setData(firstIndex(), font, Qt::FontRole);
    }
}

void XmlParser::close(const QModelIndex& index){
    Tree* fileTreeNode = rootItem->child(index.row());
    if (fileTreeNode){
        removeRow(fileTreeNode->rowInParent(), focused.parent());
        QFont font;
        font.setBold(true);
        setData(firstIndex(), font, Qt::FontRole);
    }

}

QString XmlParser::getName(){
    auto fileTreeNode = static_cast<Tree*>(focused.internalPointer());
    qDebug()<<"getName"<<fileTreeNode->data(0).toString();
    if (fileTreeNode) {
        return fileTreeNode->data(0).toString();
    }
    return "";
}

QString XmlParser::getNameByIndex(const QModelIndex& index){
    Tree* child = rootItem->child(index.row());
    if (child)
        return child->data(0).toString();
    return "";
}

XmlParser::~XmlParser() {
    delete rootItem;
    delete xmlReader;
}

QModelIndex XmlParser::firstIndex() {
    return index(0, 0);
}

bool XmlParser::removeRows(int row, int count, const QModelIndex &parent) {
    Tree* node = nullptr;
    QModelIndex correctParent;
    if (parent.isValid()) {
        node = static_cast<Tree*>(parent.internalPointer());
        correctParent = parent;
    } else {
        node = rootItem;
        correctParent = QModelIndex();
    }
    beginRemoveRows(correctParent, row, row + count - 1);
    endRemoveRows();
    return node->removeChilds(row, row + count);
}











